searchData={"items":[{"type":"module","title":"Phantom","doc":"MCP (Model Context Protocol) framework for Elixir Plug.\n\nThis library provides a complete implementation of the [MCP server specification](https://modelcontextprotocol.io/specification/2025-03-26/basic/transports) with Plug.\n\n**Only stateless is tested at this moment. Do not expect stateful (`{:noreply, ...}`) to work**","ref":"Phantom.html"},{"type":"module","title":"Usage Example - Phantom","doc":"For Streamable HTTP access to your MCP server, forward\na path from your Plug or Phoenix Router to your MCP router.\n\nFor Phoenix:\n\n```elixir\npipeline :mcp do\n  plug :accepts, [\"json\"]\n\n  plug Plug.Parsers,\n    parsers: [{:json, length: 1_000_000}],\n    pass: [\"application/json\"],\n    json_decoder: JSON\nend\n\nscope \"/mcp\" do\n  pipe_through :mcp\n\n  forward \"/\", Phantom.Plug,\n    router: MyApp.MCPRouter\nend\n```\n\nFor Plug:\n\n```elixir\ndefmodule MyAppWeb.Router do\n  use Plug.Router\n\n  plug :match\n  plug Plug.Parsers,\n    parsers: [{:json, length: 1_000_000}],\n    pass: [\"application/json\"],\n    json_decoder: JSON\n  plug :dispatch\n\n  forward \"/mcp\",\n    to: Phantom.Plug,\n    init_opts: [router: MyApp.MCP.Router]\nend\n```\n\nIn your MCP Router, define the available tooling (prompts, resources, tools) and\noptional connect and close callbacks.\n\n```elixir\ndefmodule MyApp.MCPRouter do\n  use Phantom.Router,\n    name: \"MyApp\",\n    vsn: \"1.0\"\n\n  require Logger\n\n  # recommended\n  def connect(session, _last_event_id) do\n    with {:ok, user} <- MyApp.authenticate(conn),\n         {:ok, my_session_state} <- MyApp.load_session(session.id) do\n      {:ok, assign(session, some_state: my_session_state, user: user)\n    end\n  end\n\n  # optional\n  def disconnect(session) do\n    Logger.info(\"Disconnected: #{inspect(session)}\")\n  end\n\n  # optional\n  def terminate(session) do\n    MyApp.archive_session(session.id)\n    Logger.info(\"Session completed: #{inspect(session)}\")\n  end\n\n  @description \"\"\"\n  Review the provided Study and provide meaningful feedback about the study and let me know if there are gaps or missing questions. We want\n  a meaningful study that can provide insight to the research goals stated\n  in the study.\n  \"\"\"\n  prompt :suggest_questions, MyApp.MCP,\n    description: @description,\n    completion_function: :study_id_complete,\n    arguments: [\n      %{\n        name: \"study_id\",\n        description: \"The study to review\",\n        required: true\n      }\n    ]\n\n  # Defining available resources\n  @description \"\"\"\n  Read the cover image of a Study to gain some context of the\n  audience, research goals, and questions.\n  \"\"\"\n  resource \"myapp:///studies/:study_id/cover\", MyApp.MCP, :study_cover,\n    completion_function: :study_id_complete,\n    mime_type: \"image/png\"\n\n  @description \"\"\"\n  Read the contents of a study. This includes the questions and general\n  context, which is helpful for understanding research goals.\n  \"\"\"\n  resource \"https://example.com/studies/:study_id/md\", MyApp.MCP, :study,\n    completion_function: :study_id_complete,\n    mime_type: \"text/markdown\"\n\n  # Defining available tools\n  @description \"\"\"\n  Create a question for the provided Study.\n  \"\"\"\n  tool :create_question, MyApp.MCP,\n    input_schema: %{\n      required: ~w[description label study_id],\n      properties: %{\n        study_id: %{\n          type: \"integer\",\n          description: \"The unique identifier for the Study\"\n        },\n        label: %{\n          type: \"string\",\n          description: \"The title of the Question. The first thing the participant will see when presented with the question\"\n        },\n        description: %{\n          type: \"string\",\n          description: \"The contents of the question. About one paragraph of detail that defines one question or task for the participant to perform or answer\"\n        }\n      }\n    }\n  }]\nend\n```\n\nIn the connect callback, you can limit the available tools, prompts, and resources\ndepending on authorization rules by supplying an allow list of names:\n\n```elixir\n  def connect(session, _last_event_id) do\n    with {:ok, user} <- MyApp.authenticate(session) do\n      {:ok,\n        session\n        |> assign(:user, user)\n        |> limit_for_plan(user.plan)}\n    end\n  end\n\n  defp limit_for_plan(session, :basic) do\n    # allow-list tools by name\n    %{session |\n      resources: ~w[study],\n      tools: ~w[create_question]}\n  end\n\n  defp limit_for_plan(session, :ultra), do: session\n```\n\nImplement handlers that resemble a GenServer behaviour. Each handler function\nwill receive three arguments:\n\n1. the params of the request\n2. the request\n3. the session\n\n```elixir\ndefmodule MyApp.MCP do\n  alias MyApp.Repo\n  alias MyApp.Study\n\n  import MyApp.MCPRouter, only: [resource_for: 3], warn: false\n\n  def suggest_questions(%{\"study_id\" => study_id} = _params, _request, session) do\n    case Repo.get(Study, study_id) do\n      {:reply, %{\n        role: :assistant,\n        # Can be \"text\", \"audio\", \"image\", or \"resource\"\n        type: \"text\",\n        # When referencing a resource, supply a `resource: data`\n        # You can use the imported `resource_for` helper that will\n        # construct a response object pointing to the resource.\n        # `resource: resource_for(session, :study, id: study.id)`\n        #\n        # For binary, supply  `data: base64-encoded-content`\n        #\n        # Below is an example of text content:\n        text: \"How was your day?\",\n        # mime_type can be supplied here, or the default mime_type\n        # defined along with the prompt will be used.\n        mime_type: \"text/plain\"\n      }, session}\n      _ ->\n       {:error, \"not found\"}\n    end\n  end\n\n  def study(%{\"study_id\" => id} = params, _request, session) do\n    study = Repo.get(Study, id)\n    text = Study.to_markdown(study)\n    # Must return a map with a `:text` key\n    # or a `:binary` key with base64-encoded data.\n    {:reply, %{text: text}, session}\n  end\n\n  def study_cover(%{\"study_id\" => id} = params, _request, session) do\n    study = Repo.get(Study, id)\n    binary = File.read!(study.cover.file)\n    {:reply, %{binary: Base.encode64(binary)}, session}\n  end\n\n  import Ecto.Query\n  def study_id_complete(\"study_id\", value, session) do\n    study_ids = Repo.all(\n      from s in Study,\n        select: s.id,\n        where: like(type(:id, :string), \"#{value}%\"),\n        where: s.account_id == ^session.user.account_id,\n        limit: 100\n      )\n\n    # You may also return a map with more info:\n    # `%{values: study_ids, has_more: true, total: 1_000_000}`\n    {:reply, study_ids, session}\n  end\n\n  def create_question(params, _request, session) do\n    %{\"study_id\" => study_id, \"label\" => label, \"description\" => description} = params\n\n    case Study.create_question(study_id, label: label, description: description) do\n      {:ok, question} ->\n        md = Study.Question.to_markdown(question)\n        {:reply, %{type: :text, text: md}, session}\n      _ ->\n        {:reply, %{type: :text, text: \"Could not create\", error: true}, session}\n    end\n  end\nend\n```\n\nPhantom will implement these MCP requests on your behalf:\n\n- `initialize` accessible in the `connect/2` callback\n- `prompts/list` which will list either the allowed prompts in the `connect/2` callback, or all prompts by default\n- `prompts/get` which will dispatch the request to your handler\n- `resources/list` which will list either the provided resources in the `connect/2` callback, or all resources by default\n- `resources/get` which will dispatch the request to your handler\n- `resource/templates/list` which will list available as defined in the router.\n- `tools/list` which will list either the provided tools in the `connect/2` callback, or all tools by default\n- `tools/call` which will dispatch the request to your handler\n- `completion/complete` which will dispatch the request to your handler for the given\nprompt or resource\n- `notification/*` which will be no-op.\n- `ping` pong\n\nBatched requests will also be handled transparently. **please note** there is not\nan abstraction yet for efficiently providing these as a group to your handler.\nThe plan is to dispatch the list of params to your handler, and the handler can check\nif the params are a list or not.","ref":"Phantom.html#module-usage-example"},{"type":"module","title":"Phantom.Plug","doc":"Main Plug implementation for MCP HTTP transport with SSE support.\n\n This module provides a complete MCP server implementation with:\n - JSON-RPC 2.0 message handling\n - Server-Sent Events (SSE) streaming\n - CORS handling and security features\n - Session management integration\n - Origin validation\n\n In your Phoenix router where you can accept JSON:\n\n      pipeline :mcp do\n        plug :accepts, [\"json\"]\n      end\n\n      scope \"/mcp\" do\n        pipe_through :mcp\n\n        forward \"/\", Phantom.Plug,\n          router: Test.MCPRouter,\n          validate_origin: false\n      end\n\n  For in your Plug Router after you parse the body:\n\n      use Plug.Router\n      plug :match\n      plug Plug.Parsers,\n        parsers: [{:json, length: 1_000_000}],\n        pass: [\"application/json\"],\n        json_decoder: JSON\n      plug :dispatch\n\n      forward \"/mcp\",\n        to: Phantom.Plug,\n        init_opts: [validate_origin: false, router: Test.MCPRouter]\n\nHere are the defaults:\n\n```elixir\n[\n  origins: [\"http://localhost:4000\"],\n  validate_origin: true,\n  session_timeout: 30000,\n  max_request_size: 1048576\n]\n```","ref":"Phantom.Plug.html"},{"type":"module","title":"Telemetry - Phantom.Plug","doc":"Telemetry is provided with these events:\n\n- `[:phantom, :plug, :request, :connect]` with meta: `~w[session_id last_event_id router opts conn]a`\n- `[:phantom, :plug, :request, :disconnect]` with meta: `~w[session router conn]a`\n- `[:phantom, :plug, :request, :exception]` with meta: `~w[session router conn stacktrace request exception]a`","ref":"Phantom.Plug.html#module-telemetry"},{"type":"function","title":"Phantom.Plug.call/2","doc":"Callback implementation for  Plug.call/2 .","ref":"Phantom.Plug.html#call/2"},{"type":"function","title":"Phantom.Plug.init/1","doc":"Initializes the plug with the given options.","ref":"Phantom.Plug.html#init/1"},{"type":"function","title":"Options - Phantom.Plug.init/1","doc":"- `:router` - The MCP router module (required)\n- `:origins` - List of allowed origins or `:all` (default: localhost)\n- `:validate_origin` - Whether to validate Origin header (default: true)\n- `:session_timeout` - Session timeout in milliseconds (default: 300_000)\n- `:max_request_size` - Maximum request size in bytes (default: 1MB)","ref":"Phantom.Plug.html#init/1-options"},{"type":"type","title":"Phantom.Plug.opts/0","doc":"","ref":"Phantom.Plug.html#t:opts/0"},{"type":"module","title":"Phantom.Prompt","doc":"The Model Context Protocol (MCP) provides a standardized way\nfor servers to expose prompt templates to clients. Prompts\nallow servers to provide structured messages and instructions\nfor interacting with language models. Clients can discover\navailable prompts, retrieve their contents, and provide arguments\nto customize them.\n\nhttps://modelcontextprotocol.io/specification/2025-03-26/server/prompts","ref":"Phantom.Prompt.html"},{"type":"function","title":"Phantom.Prompt.build/1","doc":"","ref":"Phantom.Prompt.html#build/1"},{"type":"type","title":"Phantom.Prompt.json/0","doc":"","ref":"Phantom.Prompt.html#t:json/0"},{"type":"type","title":"Phantom.Prompt.t/0","doc":"","ref":"Phantom.Prompt.html#t:t/0"},{"type":"function","title":"Phantom.Prompt.to_json/1","doc":"","ref":"Phantom.Prompt.html#to_json/1"},{"type":"module","title":"Phantom.Prompt.Argument","doc":"","ref":"Phantom.Prompt.Argument.html"},{"type":"function","title":"Phantom.Prompt.Argument.build/1","doc":"","ref":"Phantom.Prompt.Argument.html#build/1"},{"type":"type","title":"Phantom.Prompt.Argument.json/0","doc":"","ref":"Phantom.Prompt.Argument.html#t:json/0"},{"type":"type","title":"Phantom.Prompt.Argument.t/0","doc":"","ref":"Phantom.Prompt.Argument.html#t:t/0"},{"type":"function","title":"Phantom.Prompt.Argument.to_json/1","doc":"","ref":"Phantom.Prompt.Argument.html#to_json/1"},{"type":"module","title":"Phantom.ResourceTemplate","doc":"The Model Context Protocol (MCP) provides a standardized way for\nservers to expose resources to clients. Resources allow servers to\nshare data that provides context to language models, such as files,\ndatabase schemas, or application-specific information. Each resource\nis uniquely identified by a URI.\n\nhttps://modelcontextprotocol.io/specification/2025-03-26/server/resources","ref":"Phantom.ResourceTemplate.html"},{"type":"function","title":"Phantom.ResourceTemplate.build/1","doc":"","ref":"Phantom.ResourceTemplate.html#build/1"},{"type":"type","title":"Phantom.ResourceTemplate.json/0","doc":"","ref":"Phantom.ResourceTemplate.html#t:json/0"},{"type":"type","title":"Phantom.ResourceTemplate.t/0","doc":"","ref":"Phantom.ResourceTemplate.html#t:t/0"},{"type":"function","title":"Phantom.ResourceTemplate.to_json/1","doc":"","ref":"Phantom.ResourceTemplate.html#to_json/1"},{"type":"behaviour","title":"Phantom.Router","doc":"A DSL for defining MCP servers.\nThis module provides functions that define tools, resources, and prompts.","ref":"Phantom.Router.html"},{"type":"behaviour","title":"Usage - Phantom.Router","doc":"```elixir\ndefmodule MyApp.MCP.Router do\n  use Phantom.Router,\n    name: \"MyApp\",\n    vsn: \"1.0\"\n\n  # Call MyApp.MCP.hello/3\n  tool :hello, MyApp.MCP\n\n  # Call MyApp.MCP.Router.hello/3\n  tool :hello\n\n  # Call MyApp.MCP.code_review/3\n  prompt :code_review, MyApp.MCP\n\n  # Call MyApp.MCP.studies/3\n  resource \"my_app:///studies/:id\", MyApp.MCP, :studies,\n    name: \"Studies\",\n    mime_type: \"application/json\"\n\n  # Call MyApp.MCP.questions/3\n  resource \"my_app:///questions/:id\", MyApp.MCP, :questions,\n    name: \"Questions\",\n    mime_type: \"application/html\"\nend\n```","ref":"Phantom.Router.html#module-usage"},{"type":"behaviour","title":"Telemetry - Phantom.Router","doc":"Telemetry is provided with these events:\n\n- `[:phantom, :dispatch, :start]` with meta: `~w[method params request session]a`\n- `[:phantom, :dispatch, :stop]` with meta: `~w[method params request result session]a`\n- `[:phantom, :dispatch, :exception]` with meta: `~w[method kind reason stacktrace params request session]a`","ref":"Phantom.Router.html#module-telemetry"},{"type":"callback","title":"Phantom.Router.connect/2","doc":"","ref":"Phantom.Router.html#c:connect/2"},{"type":"callback","title":"Phantom.Router.disconnect/1","doc":"","ref":"Phantom.Router.html#c:disconnect/1"},{"type":"callback","title":"Phantom.Router.dispatch_method/4","doc":"","ref":"Phantom.Router.html#c:dispatch_method/4"},{"type":"callback","title":"Phantom.Router.instructions/1","doc":"","ref":"Phantom.Router.html#c:instructions/1"},{"type":"callback","title":"Phantom.Router.list_resources/3","doc":"","ref":"Phantom.Router.html#c:list_resources/3"},{"type":"macro","title":"Phantom.Router.prompt/2","doc":"","ref":"Phantom.Router.html#prompt/2"},{"type":"macro","title":"Phantom.Router.prompt/3","doc":"Define a prompt that can be retrieved by the MCP client.","ref":"Phantom.Router.html#prompt/3"},{"type":"macro","title":"Examples - Phantom.Router.prompt/3","doc":"prompt :summarize,\n      description: \"A text prompt\",\n      completion_function: :summarize_complete,\n      arguments: [\n        %{\n          name: \"text\",\n          description: \"The text to summarize\",\n        },\n        %{\n          name: \"resource\",\n          description: \"The resource to summarize\",\n        }\n      ]\n    )\n\n    # ...\n\n    def summarize(args, _request, session) do\n      {:reply, %{}, session}\n    end\n\n    def summarize_complete(\"text\", _, session) do\n      {:reply, [], session}\n    end\n\n    def summarize_complete(\"resource\", _, session) do\n      # list of IDs\n      {:reply, [\"123\"], session}\n    end","ref":"Phantom.Router.html#prompt/3-examples"},{"type":"macro","title":"Phantom.Router.resource/4","doc":"Define a resource that can be read by the MCP client.","ref":"Phantom.Router.html#resource/4"},{"type":"macro","title":"Examples - Phantom.Router.resource/4","doc":"resource \"app:///studies/:id\", MyApp.MCP, :read_study,\n      description: \"A study\",\n      mime_type: \"application/json\"\n\n    # ...\n\n    def read_study(%{\"id\" => id}, _request, session) do\n      {:reply, %{\n        uri: \"file:///project/lib/application.ex\",\n        mime_type: \"text/x-elixir\",\n        text: \"IO.puts \"Hi\"\"\n      }, session}\n    end","ref":"Phantom.Router.html#resource/4-examples"},{"type":"function","title":"Phantom.Router.resource_for/3","doc":"Constructs a response map for the given resource with the provided parameters. This\nfunction is provided to your MCP Router that accepts the session instead.\n\nFor example\n\n```elixir\niex> MyMCPRouter.resource_for(session, :name_of_resource, id: 123)\n%{\n  uri: \"myapp:///my-resource/123\",\n  mimeType: \"application/json\"\n  text: \"name of my resource\"\n}\n```","ref":"Phantom.Router.html#resource_for/3"},{"type":"callback","title":"Phantom.Router.server_info/1","doc":"","ref":"Phantom.Router.html#c:server_info/1"},{"type":"callback","title":"Phantom.Router.terminate/1","doc":"","ref":"Phantom.Router.html#c:terminate/1"},{"type":"macro","title":"Phantom.Router.tool/2","doc":"","ref":"Phantom.Router.html#tool/2"},{"type":"macro","title":"Phantom.Router.tool/3","doc":"Define a tool that can be called by the MCP client.","ref":"Phantom.Router.html#tool/3"},{"type":"macro","title":"Examples - Phantom.Router.tool/3","doc":"tool :local_echo,\n      description: \"A test that echos your message\",\n      input_schema: %{\n        required: [:message],\n        properties: %{\n          message: %{\n            type: \"string\",\n            description: \"message to echo\"\n          }\n        }\n      }\n\n    ###\n\n    def local_echo(params, _request, session) do\n      {:reply, %{type: \"text\", text: params[\"message\"]}, session}\n    end","ref":"Phantom.Router.html#tool/3-examples"},{"type":"module","title":"Phantom.Session","doc":"Represents the state of the MCP session. This is the state across the conversation\nand is the bridge between the various transports (HTTP, stdio) to persistence,\neven if stateless.","ref":"Phantom.Session.html"},{"type":"function","title":"Phantom.Session.assign/2","doc":"","ref":"Phantom.Session.html#assign/2"},{"type":"function","title":"Phantom.Session.assign/3","doc":"","ref":"Phantom.Session.html#assign/3"},{"type":"function","title":"Phantom.Session.new/3","doc":"","ref":"Phantom.Session.html#new/3"},{"type":"type","title":"Phantom.Session.t/0","doc":"","ref":"Phantom.Session.html#t:t/0"},{"type":"module","title":"Phantom.Tool","doc":"The Model Context Protocol (MCP) allows servers to expose tools\nthat can be invoked by language models. Tools enable models to\ninteract with external systems, such as querying databases,\ncalling APIs, or performing computations. Each tool is uniquely\nidentified by a name and includes metadata describing its schema.\n\nhttps://modelcontextprotocol.io/specification/2025-03-26/server/tools","ref":"Phantom.Tool.html"},{"type":"function","title":"Phantom.Tool.build/1","doc":"","ref":"Phantom.Tool.html#build/1"},{"type":"type","title":"Phantom.Tool.json/0","doc":"","ref":"Phantom.Tool.html#t:json/0"},{"type":"type","title":"Phantom.Tool.t/0","doc":"","ref":"Phantom.Tool.html#t:t/0"},{"type":"function","title":"Phantom.Tool.to_json/1","doc":"","ref":"Phantom.Tool.html#to_json/1"},{"type":"module","title":"Phantom.Tool.Annotation","doc":"Tool annotations provide additional metadata about a\ntool’s behavior, helping clients understand how to present\nand manage tools. These annotations are hints that describe\nthe nature and impact of a tool, but should not be relied\nupon for security decisions\n\n- `:title` A human-readable title for the tool, useful for UI display\n- `:read_only_hint` If true, indicates the tool does not modify its environment\n- `:destructive_hint` If true, the tool may perform destructive updates (only meaningful when `:read_only_hint` is false)\n- `:idempotent_hint` If true, calling the tool repeatedly with the same arguments has no additional effect (only meaningful when readOnlyHint is false)\n- `:open_world_hint` If true, the tool may interact with an “open world” of external entities\n\nhttps://modelcontextprotocol.io/docs/concepts/tools#tool-annotations","ref":"Phantom.Tool.Annotation.html"},{"type":"function","title":"Phantom.Tool.Annotation.build/1","doc":"","ref":"Phantom.Tool.Annotation.html#build/1"},{"type":"type","title":"Phantom.Tool.Annotation.json/0","doc":"","ref":"Phantom.Tool.Annotation.html#t:json/0"},{"type":"type","title":"Phantom.Tool.Annotation.t/0","doc":"","ref":"Phantom.Tool.Annotation.html#t:t/0"},{"type":"function","title":"Phantom.Tool.Annotation.to_json/1","doc":"","ref":"Phantom.Tool.Annotation.html#to_json/1"},{"type":"module","title":"Phantom.Tool.InputSchema","doc":"JSON Schema representing the arguments for the tool\n\nLearn more at   https://json-schema.org/learn/getting-started-step-by-step\n\nExample:\n\n    %{\n      type: \"object\",\n      properties: %{\n        productId: %{\n          description: \"The unique identifier for a product\",\n          type: \"integer\"\n        },\n        productName: %{\n          description: \"Name of the product\",\n          type: \"string\"\n        }\n      }\n    }","ref":"Phantom.Tool.InputSchema.html"},{"type":"function","title":"Phantom.Tool.InputSchema.build/1","doc":"","ref":"Phantom.Tool.InputSchema.html#build/1"},{"type":"type","title":"Phantom.Tool.InputSchema.json/0","doc":"","ref":"Phantom.Tool.InputSchema.html#t:json/0"},{"type":"type","title":"Phantom.Tool.InputSchema.t/0","doc":"","ref":"Phantom.Tool.InputSchema.html#t:t/0"},{"type":"function","title":"Phantom.Tool.InputSchema.to_json/1","doc":"","ref":"Phantom.Tool.InputSchema.html#to_json/1"},{"type":"exception","title":"Phantom.ErrorWrapper","doc":"Wraps errors that occur during a request or batch or requests.\nThis allows the connection to finish, and then reraises with this error\ncontaining the exceptions by request.","ref":"Phantom.ErrorWrapper.html"},{"type":"function","title":"Phantom.ErrorWrapper.new/2","doc":"","ref":"Phantom.ErrorWrapper.html#new/2"},{"type":"extras","title":"CHANGELOG","doc":"","ref":"changelog.html"},{"type":"extras","title":"Unreleased - CHANGELOG","doc":"Work in progress","ref":"changelog.html#unreleased"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}